@@ -15,6 +15,16 @@ http://media.example.com/entire.ts
 #EXT-X-ENDLIST
 '''
 
+SIMPLE_PLAYLIST_WITH_ZERO_DURATION = '''
+#EXTM3U
+#EXT-X-TARGETDURATION:5220
+#EXTINF:0,
+http://media.example.com/entire1.ts
+#EXTINF:5220,
+http://media.example.com/entire2.ts
+#EXT-X-ENDLIST
+'''
+
 SIMPLE_PLAYLIST_WITH_START_NEGATIVE_OFFSET = '''
 #EXTM3U
 #EXT-X-TARGETDURATION:5220
@@ -61,7 +71,7 @@ https://priv.example.com/fileSequence2681.ts
 https://priv.example.com/fileSequence2682.ts
 '''
 
-PLAYLIST_WITH_ENCRIPTED_SEGMENTS = '''
+PLAYLIST_WITH_ENCRYPTED_SEGMENTS = '''
 #EXTM3U
 #EXT-X-MEDIA-SEQUENCE:7794
 #EXT-X-TARGETDURATION:15
@@ -76,6 +86,21 @@ http://media.example.com/fileSequence52-2.ts
 http://media.example.com/fileSequence52-3.ts
 '''
 
+PLAYLIST_WITH_SESSION_ENCRYPTED_SEGMENTS = '''
+#EXTM3U
+#EXT-X-MEDIA-SEQUENCE:7794
+#EXT-X-TARGETDURATION:15
+
+#EXT-X-SESSION-KEY:METHOD=AES-128,URI="https://priv.example.com/key.php?r=52"
+
+#EXTINF:15,
+http://media.example.com/fileSequence52-1.ts
+#EXTINF:15,
+http://media.example.com/fileSequence52-2.ts
+#EXTINF:15,
+http://media.example.com/fileSequence52-3.ts
+'''
+
 VARIANT_PLAYLIST = '''
 #EXTM3U
 #EXT-X-STREAM-INF:PROGRAM-ID=1, BANDWIDTH=1280000
@@ -217,7 +242,7 @@ segment.ts
 #EXT-X-ENDLIST
 '''
 
-PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV = '''
+PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV = '''
 #EXTM3U
 #EXT-X-MEDIA-SEQUENCE:82400
 #EXT-X-ALLOW-CACHE:NO
@@ -238,7 +263,28 @@ PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV = '''
 ../../../../hls/streamNum82405.ts
 '''
 
-PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED = '''
+PLAYLIST_WITH_SESSION_ENCRYPTED_SEGMENTS_AND_IV = '''
+#EXTM3U
+#EXT-X-MEDIA-SEQUENCE:82400
+#EXT-X-ALLOW-CACHE:NO
+#EXT-X-VERSION:2
+#EXT-X-SESSION-KEY:METHOD=AES-128,URI="/hls-key/key.bin", IV=0X10ef8f758ca555115584bb5b3c687f52
+#EXT-X-TARGETDURATION:8
+#EXTINF:8,
+../../../../hls/streamNum82400.ts
+#EXTINF:8,
+../../../../hls/streamNum82401.ts
+#EXTINF:8,
+../../../../hls/streamNum82402.ts
+#EXTINF:8,
+../../../../hls/streamNum82403.ts
+#EXTINF:8,
+../../../../hls/streamNum82404.ts
+#EXTINF:8,
+../../../../hls/streamNum82405.ts
+'''
+
+PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_SORTED = '''
 #EXTM3U
 #EXT-X-MEDIA-SEQUENCE:82400
 #EXT-X-ALLOW-CACHE:NO
@@ -259,7 +305,28 @@ PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED = '''
 ../../../../hls/streamNum82405.ts
 '''
 
-PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS = '''
+PLAYLIST_WITH_SESSION_ENCRYPTED_SEGMENTS_AND_IV_SORTED = '''
+#EXTM3U
+#EXT-X-MEDIA-SEQUENCE:82400
+#EXT-X-ALLOW-CACHE:NO
+#EXT-X-VERSION:2
+#EXT-X-TARGETDURATION:8
+#EXT-X-SESSION-KEY:METHOD=AES-128,URI="/hls-key/key.bin", IV=0X10ef8f758ca555115584bb5b3c687f52
+#EXTINF:8,
+../../../../hls/streamNum82400.ts
+#EXTINF:8,
+../../../../hls/streamNum82401.ts
+#EXTINF:8,
+../../../../hls/streamNum82402.ts
+#EXTINF:8,
+../../../../hls/streamNum82403.ts
+#EXTINF:8,
+../../../../hls/streamNum82404.ts
+#EXTINF:8,
+../../../../hls/streamNum82405.ts
+'''
+
+PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS = '''
 #EXTM3U
 #EXT-X-MEDIA-SEQUENCE:82400
 #EXT-X-ALLOW-CACHE:NO
@@ -281,7 +348,7 @@ PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS = '''
 ../../../../hls/streamNum82405.ts
 '''
 
-PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS_SORTED = '''
+PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS_SORTED = '''
 #EXTM3U
 #EXT-X-MEDIA-SEQUENCE:82400
 #EXT-X-ALLOW-CACHE:NO
@@ -420,8 +487,15 @@ PLAYLIST_WITH_MULTIPLE_KEYS_UNENCRYPTED_AND_ENCRYPTED_NONE_AND_NO_URI_ATTR = '''
 
 '''
 
+PLAYLIST_WITH_KEYFORMAT_AND_KEYFORMATVERSIONS='''#EXTM3U
+#EXT-X-VERSION:5
+#EXT-X-TARGETDURATION:8
+#EXT-X-KEY:METHOD=SAMPLE-AES,URI="skd://someuri",KEYFORMAT="com.apple.streamingkeydelivery",KEYFORMATVERSIONS="1"
+#EXTINF:8,
+segment.ts
+'''
 
-SIMPLE_PLAYLIST_WITH_TITLE = '''
+SIMPLE_PLAYLIST_WITH_QUOTED_TITLE = '''
 #EXTM3U
 #EXT-X-TARGETDURATION:5220
 #EXTINF:5220,"A sample title"
@@ -429,6 +503,14 @@ http://media.example.com/entire.ts
 #EXT-X-ENDLIST
 '''
 
+SIMPLE_PLAYLIST_WITH_UNQUOTED_TITLE = '''
+#EXTM3U
+#EXT-X-TARGETDURATION:5220
+#EXTINF:5220,A sample unquoted title
+http://media.example.com/entire.ts
+#EXT-X-ENDLIST
+'''
+
 SIMPLE_PLAYLIST_WITH_RESOLUTION = '''
 #EXTM3U
 #EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=445000,RESOLUTION=512x288,CODECS="avc1.77.30, mp4a.40.5"
@@ -643,10 +725,42 @@ CUE_OUT_ENVIVIO_PLAYLIST = '''
 20160914T080055-master804-199/1710.ts
 '''
 
+CUE_OUT_INVALID_PLAYLIST = '''#EXTM3U
+#EXT-X-TARGETDURATION:6
+#EXT-X-CUE-OUT:INVALID
+#EXTINF:5.76, no desc
+0.aac
+'''
+
+CUE_OUT_NO_DURATION_PLAYLIST = '''#EXTM3U
+#EXT-X-TARGETDURATION:6
+#EXT-X-CUE-OUT
+#EXTINF:5.76,
+0.aac
+#EXTINF:5.76,
+1.aac
+#EXT-X-CUE-IN
+#EXTINF:5.76,
+2.aac
+'''
+
+CUE_OUT_WITH_DURATION_PLAYLIST = '''#EXTM3U
+#EXT-X-TARGETDURATION:6
+#EXT-X-CUE-OUT:11.52
+#EXTINF:5.76,
+0.aac
+#EXTINF:5.76,
+1.aac
+#EXT-X-CUE-IN
+#EXTINF:5.76,
+2.aac
+'''
+
 MULTI_MEDIA_PLAYLIST = '''#EXTM3U
 #EXT-X-VERSION:3
 #EXT-X-MEDIA:URI="chinese/ed.ttml",TYPE=SUBTITLES,GROUP-ID="subs",LANGUAGE="zho",NAME="Chinese",AUTOSELECT=YES,FORCED=NO
-#EXT-X-MEDIA:URI="french/ed.ttml",TYPE=SUBTITLES,GROUP-ID="subs",LANGUAGE="fra",NAME="French",AUTOSELECT=YES,FORCED=NO
+#EXT-X-MEDIA:URI="french/ed.ttml",TYPE=SUBTITLES,GROUP-ID="subs",LANGUAGE="fra",ASSOC-LANGUAGE="fra",NAME="French",AUTOSELECT=YES,FORCED=NO,CHARACTERISTICS="public.accessibility.transcribes-spoken-dialog,public.accessibility.describes-music-and-sound"
+#EXT-X-MEDIA:TYPE=CLOSED-CAPTIONS,GROUP-ID="cc",LANGUAGE="sp",NAME="CC2",AUTOSELECT=YES,INSTREAM-ID="CC2"
 #EXT-X-MEDIA:URI="en/chunklist_w370587926_b160000_ao_slen_t64RW5nbGlzaA==.m3u8",TYPE=AUDIO,GROUP-ID="aac",LANGUAGE="en",NAME="English",DEFAULT=YES,AUTOSELECT=YES
 #EXT-X-MEDIA:URI="sp/chunklist_w370587926_b160000_ao_slsp_t64U3BhbmlzaA==.m3u8",TYPE=AUDIO,GROUP-ID="aac",LANGUAGE="sp",NAME="Spanish",DEFAULT=NO,AUTOSELECT=YES
 #EXT-X-MEDIA:URI="com/chunklist_w370587926_b160000_ao_slen_t64Q29tbWVudGFyeSAoZW5nKQ==.m3u8",TYPE=AUDIO,GROUP-ID="aac",LANGUAGE="en",NAME="Commentary (eng)",DEFAULT=NO,AUTOSELECT=NO
@@ -674,6 +788,25 @@ MAP_URI_PLAYLIST_WITH_BYTERANGE = '''#EXTM3U
 #EXT-X-PLAYLIST-TYPE:VOD
 #EXT-X-INDEPENDENT-SEGMENTS
 #EXT-X-MAP:URI="main.mp4",BYTERANGE="812@0"
+#EXTINF:1,
+segment_link.mp4
+'''
+
+MULTIPLE_MAP_URI_PLAYLIST = '''#EXTM3U
+#EXT-X-TARGETDURATION:6
+#EXT-X-VERSION:7
+#EXT-X-MEDIA-SEQUENCE:1
+#EXT-X-PLAYLIST-TYPE:VOD
+#EXT-X-INDEPENDENT-SEGMENTS
+#EXT-X-KEY:URI="key.bin",METHOD=AES-128
+#EXT-X-MAP:URI="init1.mp4"
+#EXTINF:5,
+segment1.mp4
+#EXTINF:5,
+segment2.mp4
+#EXT-X-MAP:URI="init3.mp4"
+#EXTINF:5,
+segment3.mp4
 '''
 
 MEDIA_WITHOUT_URI_PLAYLIST = '''#EXTM3U
@@ -699,6 +832,56 @@ http://media.example.com/entire.ts
 #EXT-X-ENDLIST
 '''
 
+LOW_LATENCY_DELTA_UPDATE_PLAYLIST = '''#EXTM3U
+# Following the example above, this playlist is a response to: GET https://example.com/2M/waitForMSN.php?_HLS_msn=273&_HLS_part=3&_HLS_report=../1M/waitForMSN.php&_HLS_report=../4M/waitForMSN.php&_HLS_skip=YES
+#EXT-X-TARGETDURATION:4
+#EXT-X-VERSION:9
+#EXT-X-SERVER-CONTROL:CAN-BLOCK-RELOAD=YES,PART-HOLD-BACK=1.0,CAN-SKIP-UNTIL=12.0
+#EXT-X-PART-INF:PART-TARGET=0.33334
+#EXT-X-MEDIA-SEQUENCE:266
+#EXT-X-SKIP:SKIPPED-SEGMENTS=3
+#EXTINF:4.00008,
+fileSequence269.ts
+#EXTINF:4.00008,
+fileSequence270.ts
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.0.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.1.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.2.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.3.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.4.ts",INDEPENDENT=YES
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.5.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.6.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.7.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.8.ts",INDEPENDENT=YES
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.9.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.10.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart271.11.ts"
+#EXTINF:4.00008,
+fileSequence271.ts
+#EXT-X-PROGRAM-DATE-TIME:2019-02-14T02:14:00.106Z
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.a.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.b.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.c.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.d.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.e.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.f.ts",INDEPENDENT=YES
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.g.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.h.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.i.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.j.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.k.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart272.l.ts"
+#EXTINF:4.00008,
+fileSequence272.ts
+#EXT-X-PART:DURATION=0.33334,URI="filePart273.0.ts",INDEPENDENT=YES
+#EXT-X-PART:DURATION=0.33334,URI="filePart273.1.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart273.2.ts"
+#EXT-X-PART:DURATION=0.33334,URI="filePart273.3.ts"
+
+#EXT-X-RENDITION-REPORT:URI="../1M/waitForMSN.php",LAST-MSN=273,LAST-PART=3
+#EXT-X-RENDITION-REPORT:URI="../4M/waitForMSN.php",LAST-MSN=273,LAST-PART=3
+'''
+
 RELATIVE_PLAYLIST_FILENAME = abspath(join(dirname(__file__), 'playlists/relative-playlist.m3u8'))
 
 RELATIVE_PLAYLIST_URI = TEST_HOST + '/path/to/relative-playlist.m3u8'
@@ -719,4 +902,30 @@ http://example.com/hi.m3u8
 http://example.com/audio-only.m3u8
 '''
 
+SESSION_DATA_PLAYLIST = '''#EXTM3U
+#EXT-X-VERSION:4
+#EXT-X-SESSION-DATA:DATA-ID="com.example.value",VALUE="example",LANGUAGE="en"
+'''
+
+MULTIPLE_SESSION_DATA_PLAYLIST = '''#EXTM3U
+#EXT-X-VERSION:4
+#EXT-X-SESSION-DATA:DATA-ID="com.example.value",VALUE="example",LANGUAGE="en"
+#EXT-X-SESSION-DATA:DATA-ID="com.example.value",VALUE="example",LANGUAGE="ru"
+#EXT-X-SESSION-DATA:DATA-ID="com.example.value",VALUE="example",LANGUAGE="de"
+#EXT-X-SESSION-DATA:DATA-ID="com.example.title",URI="title.json"
+'''
+
+VERSION_PLAYLIST = '''#EXTM3U
+#EXT-X-VERSION:4
+'''
+
+PLAYLIST_WITH_NEGATIVE_MEDIA_SEQUENCE = '''
+#EXTM3U
+#EXT-X-TARGETDURATION:5220
+#EXT-X-MEDIA-SEQUENCE:-2680
+#EXTINF:5220,
+http://media.example.com/entire.ts
+#EXT-X-ENDLIST
+'''
+
 del abspath, dirname, join
