@@ -7,12 +7,15 @@
 # data returned from parser.parse()
 
 import datetime
+import os
+import pytest
+import sys
 
-from m3u8.protocol import ext_x_start
+from m3u8.protocol import ext_x_start, ext_x_part
 
 import m3u8
 import playlists
-from m3u8.model import Segment, Key, Media
+from m3u8.model import Segment, Key, Media, MediaList, RenditionReport, PartialSegment, denormalize_attribute, find_key, SessionData
 
 
 class UTC(datetime.tzinfo):
@@ -121,6 +124,43 @@ def test_segment_cue_out_attribute():
     assert segments[2].cue_out == True
     assert segments[3].cue_out == False
 
+def test_segment_cue_out_start_attribute():
+    obj = m3u8.M3U8(playlists.CUE_OUT_WITH_DURATION_PLAYLIST)
+
+    assert obj.segments[0].cue_out_start == True
+
+def test_segment_cue_in_attribute():
+    obj = m3u8.M3U8(playlists.CUE_OUT_WITH_DURATION_PLAYLIST)
+
+    assert obj.segments[2].cue_in == True
+
+def test_segment_cue_out_cont_dumps():
+    obj = m3u8.M3U8(playlists.CUE_OUT_PLAYLIST)
+
+    result = obj.dumps()
+    expected = '#EXT-X-CUE-OUT-CONT\n'
+    assert expected in result
+
+def test_segment_cue_out_start_dumps():
+    obj = m3u8.M3U8(playlists.CUE_OUT_WITH_DURATION_PLAYLIST)
+
+    result = obj.dumps()
+    expected = '#EXT-X-CUE-OUT:11.52\n'
+    assert expected in result
+
+def test_segment_cue_out_start_no_duration_dumps():
+    obj = m3u8.M3U8(playlists.CUE_OUT_NO_DURATION_PLAYLIST)
+
+    result = obj.dumps()
+    expected = '#EXT-X-CUE-OUT\n'
+    assert expected in result
+
+def test_segment_cue_out_in_dumps():
+    obj = m3u8.M3U8(playlists.CUE_OUT_NO_DURATION_PLAYLIST)
+
+    result = obj.dumps()
+    expected = '#EXT-X-CUE-IN\n'
+    assert expected in result
 
 def test_segment_elemental_scte35_attribute():
     obj = m3u8.M3U8(playlists.CUE_OUT_ELEMENTAL_PLAYLIST)
@@ -137,6 +177,16 @@ def test_segment_envivio_scte35_attribute():
     assert segments[5].scte35 == '/DAlAAAENOOQAP/wFAUBAABrf+//N25XDf4B9p/gAAEBAQAAxKni9A=='
     assert segments[7].cue_out == False
 
+def test_segment_unknown_scte35_attribute():
+    obj = m3u8.M3U8(playlists.CUE_OUT_INVALID_PLAYLIST)
+    assert obj.segments[0].scte35 == None
+    assert obj.segments[0].scte35_duration == None
+
+def test_segment_cue_out_no_duration():
+    obj = m3u8.M3U8(playlists.CUE_OUT_NO_DURATION_PLAYLIST)
+    assert obj.segments[0].cue_out_start == True
+    assert obj.segments[2].cue_in == True
+
 def test_keys_on_clear_playlist():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
 
@@ -145,7 +195,7 @@ def test_keys_on_clear_playlist():
 
 
 def test_keys_on_simple_encrypted_playlist():
-    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS)
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS)
 
     assert len(obj.keys) == 1
     assert obj.keys[0].uri == "https://priv.example.com/key.php?r=52"
@@ -181,6 +231,48 @@ def test_key_attribute_without_initialization_vector():
     assert None == obj.keys[0].iv
 
 
+def test_session_keys_on_clear_playlist():
+    obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
+
+    assert len(obj.session_keys) == 0
+
+def test_session_keys_on_simple_encrypted_playlist():
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_SESSION_ENCRYPTED_SEGMENTS)
+
+    assert len(obj.session_keys) == 1
+    assert obj.session_keys[0].uri == "https://priv.example.com/key.php?r=52"
+
+
+def test_session_key_attribute():
+    obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
+    data = {'session_keys': [{'method': 'AES-128',
+                    'uri': '/key',
+                    'iv': 'foobar'}]}
+    mock_parser_data(obj, data)
+
+    assert 'SessionKey' == obj.session_keys[0].__class__.__name__
+    assert 'AES-128' == obj.session_keys[0].method
+    assert '/key' == obj.session_keys[0].uri
+    assert 'foobar' == obj.session_keys[0].iv
+
+
+def test_session_key_attribute_on_none():
+    obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
+    mock_parser_data(obj, {})
+
+    assert len(obj.session_keys) == 0
+
+
+def test_session_key_attribute_without_initialization_vector():
+    obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
+    mock_parser_data(obj, {'session_keys': [{'method': 'AES-128',
+                                   'uri': '/key'}]})
+
+    assert 'AES-128' == obj.session_keys[0].method
+    assert '/key' == obj.session_keys[0].uri
+    assert None == obj.session_keys[0].iv
+
+
 def test_segments_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'segments': [{'uri': '/foo/bar-1.ts',
@@ -237,7 +329,7 @@ def test_segments_attribute_with_byterange():
 
 
 def test_segment_attribute_with_multiple_keys():
-    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS)
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS)
 
     segments = obj.segments
     assert segments[0].key.uri == '/hls-key/key.bin'
@@ -245,6 +337,13 @@ def test_segment_attribute_with_multiple_keys():
     assert segments[4].key.uri == '/hls-key/key2.bin'
     assert segments[5].key.uri == '/hls-key/key2.bin'
 
+def test_segment_title_dumps():
+    obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_QUOTED_TITLE)
+
+    result = obj.segments[0].dumps(None).strip()
+    expected = '#EXTINF:5220,"A sample title"\nhttp://media.example.com/entire.ts'
+
+    assert result == expected
 
 def test_is_variant_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
@@ -385,12 +484,23 @@ def test_iframe_playlists_attribute():
 
 def test_version_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
-    mock_parser_data(obj, {'version': '2'})
-    assert '2' == obj.version
+    mock_parser_data(obj, {'version': 2})
+    assert 2 == obj.version
 
     mock_parser_data(obj, {})
     assert None == obj.version
 
+def test_version_settable_as_int():
+    obj = m3u8.loads(playlists.VERSION_PLAYLIST)
+    obj.version = 9
+
+    assert "#EXT-X-VERSION:9" in obj.dumps().strip()
+
+def test_version_settable_as_string():
+    obj = m3u8.loads(playlists.VERSION_PLAYLIST)
+    obj.version = '9'
+
+    assert "#EXT-X-VERSION:9" in obj.dumps().strip()
 
 def test_allow_cache_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
@@ -402,7 +512,7 @@ def test_allow_cache_attribute():
 
 
 def test_files_attribute_should_list_all_files_including_segments_and_key():
-    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS)
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS)
     files = [
         'https://priv.example.com/key.php?r=52',
         'http://media.example.com/fileSequence52-1.ts',
@@ -446,9 +556,9 @@ def test_dump_playlists_with_resolution():
 
 
 def test_dump_should_build_file_with_same_content(tmpdir):
-    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV)
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED.replace(', IV', ',IV').strip()
+    expected = playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_SORTED.replace(', IV', ',IV').strip()
     filename = str(tmpdir.join('playlist.m3u8'))
 
     obj.dump(filename)
@@ -457,15 +567,30 @@ def test_dump_should_build_file_with_same_content(tmpdir):
 
 
 def test_dump_should_create_sub_directories(tmpdir):
-    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV)
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED.replace(', IV', ',IV').strip()
+    expected = playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_SORTED.replace(', IV', ',IV').strip()
     filename = str(tmpdir.join('subdir1', 'subdir2', 'playlist.m3u8'))
 
     obj.dump(filename)
 
     assert_file_content(filename, expected)
 
+def test_dump_should_raise_if_create_sub_directories_fails(tmpdir, monkeypatch):
+    def raiseOSError(*args):
+        raise OSError
+
+    monkeypatch.setattr(os, "makedirs", raiseOSError)
+
+    raised = False
+    try:
+        obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
+        obj.dump(str(tmpdir.join('subdir1', 'playlist.m3u8')))
+    except OSError as e:
+        raised = True
+    finally:
+        assert raised
+
 
 def test_dump_should_work_for_variant_streams():
     obj = m3u8.M3U8(playlists.VARIANT_PLAYLIST)
@@ -513,6 +638,13 @@ def test_dump_should_include_program_date_time():
     assert "EXT-X-PROGRAM-DATE-TIME:2014-08-13T13:36:33+00:00" in obj.dumps().strip()
 
 
+def test_dump_should_not_ignore_zero_duration():
+    obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_ZERO_DURATION)
+
+    assert "EXTINF:0" in obj.dumps().strip()
+    assert "EXTINF:5220" in obj.dumps().strip()
+
+
 def test_dump_should_include_segment_level_program_date_time():
     obj = m3u8.M3U8(playlists.DISCONTINUITY_PLAYLIST_WITH_PROGRAM_DATE_TIME)
     # Tag being expected is in the segment level, not the global one
@@ -533,6 +665,24 @@ def test_dump_should_include_map_attributes():
 
     assert 'EXT-X-MAP:URI="main.mp4",BYTERANGE="812@0"' in obj.dumps().strip()
 
+
+def test_multiple_map_attributes():
+    obj = m3u8.M3U8(playlists.MULTIPLE_MAP_URI_PLAYLIST)
+
+    assert obj.segments[0].init_section.uri == 'init1.mp4'
+    assert obj.segments[1].init_section.uri == 'init1.mp4'
+    assert obj.segments[2].init_section.uri == 'init3.mp4'
+
+
+def test_dump_should_include_multiple_map_attributes():
+    obj = m3u8.M3U8(playlists.MULTIPLE_MAP_URI_PLAYLIST)
+
+    output = obj.dump('/tmp/d.m3u8')
+    output = obj.dumps().strip()
+    assert output.count('#EXT-X-MAP:URI="init1.mp4"') == 1
+    assert output.count('#EXT-X-MAP:URI="init3.mp4"') == 1
+
+
 def test_dump_should_work_for_playlists_using_byteranges():
     obj = m3u8.M3U8(playlists.PLAYLIST_USING_BYTERANGES)
 
@@ -559,8 +709,8 @@ def test_should_dump_without_endlist_tag():
 
 
 def test_should_dump_multiple_keys():
-    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS)
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS_SORTED.strip()
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS)
+    expected = playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS_SORTED.strip()
 
     assert expected == obj.dumps().strip()
 
@@ -587,6 +737,18 @@ def test_should_dump_complex_unencrypted_encrypted_keys_no_uri_attr():
 
     assert expected == obj.dumps().strip()
 
+def test_should_dump_session_data():
+    obj = m3u8.M3U8(playlists.SESSION_DATA_PLAYLIST)
+    expected = playlists.SESSION_DATA_PLAYLIST.strip()
+
+    assert expected == obj.dumps().strip()
+
+def test_should_dump_multiple_session_data():
+    obj = m3u8.M3U8(playlists.MULTIPLE_SESSION_DATA_PLAYLIST)
+    expected = playlists.MULTIPLE_SESSION_DATA_PLAYLIST.strip()
+
+    assert expected == obj.dumps().strip()
+
 
 
 def test_length_segments_by_key():
@@ -626,6 +788,13 @@ def test_replace_segment_key():
 
     assert obj.dumps().strip() == expected
 
+def test_keyformat_and_keyformatversion():
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_KEYFORMAT_AND_KEYFORMATVERSIONS)
+
+    result = obj.dumps().strip()
+    expected = 'KEYFORMAT="com.apple.streamingkeydelivery",KEYFORMATVERSIONS="1"'
+
+    assert expected in result
 
 def test_should_dump_program_datetime_and_discontinuity():
     obj = m3u8.M3U8(playlists.DISCONTINUITY_PLAYLIST_WITH_PROGRAM_DATE_TIME)
@@ -637,9 +806,26 @@ def test_should_dump_program_datetime_and_discontinuity():
 def test_should_normalize_segments_and_key_urls_if_base_path_passed_to_constructor():
     base_path = 'http://videoserver.com/hls/live'
 
-    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV, base_path)
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV, base_path)
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED \
+    assert obj.base_path == base_path
+
+    expected = playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_SORTED \
+        .replace(', IV', ',IV') \
+        .replace('../../../../hls', base_path) \
+        .replace('/hls-key', base_path) \
+        .strip()
+
+    assert obj.dumps().strip() == expected
+
+def test_should_normalize_session_key_urls_if_base_path_passed_to_constructor():
+    base_path = 'http://videoserver.com/hls/live'
+
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_SESSION_ENCRYPTED_SEGMENTS_AND_IV, base_path)
+
+    assert obj.base_path == base_path
+
+    expected = playlists.PLAYLIST_WITH_SESSION_ENCRYPTED_SEGMENTS_AND_IV_SORTED \
         .replace(', IV', ',IV') \
         .replace('../../../../hls', base_path) \
         .replace('/hls-key', base_path) \
@@ -663,10 +849,10 @@ def test_should_normalize_variant_streams_urls_if_base_path_passed_to_constructo
 def test_should_normalize_segments_and_key_urls_if_base_path_attribute_updated():
     base_path = 'http://videoserver.com/hls/live'
 
-    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV)
     obj.base_path = base_path     # update later
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED \
+    expected = playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_SORTED \
         .replace(', IV', ',IV') \
         .replace('../../../../hls', base_path) \
         .replace('/hls-key', base_path) \
@@ -678,10 +864,10 @@ def test_should_normalize_segments_and_key_urls_if_base_path_attribute_updated()
 def test_should_normalize_segments_and_key_urls_if_base_path_attribute_updated():
     base_path = 'http://videoserver.com/hls/live'
 
-    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV)
     obj.base_path = base_path
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED \
+    expected = playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_SORTED \
         .replace(', IV', ',IV') \
         .replace('../../../../hls', base_path) \
         .replace('/hls-key', base_path) \
@@ -743,12 +929,22 @@ def test_none_discontinuity_sequence_gracefully_ignored():
     expected = '#EXTM3U\n'
     assert result == expected
 
+def test_non_zero_discontinuity_sequence_added_to_file():
+    obj = m3u8.M3U8()
+    obj.discontinuity_sequence = 1
+    result = obj.dumps()
+    expected = '#EXT-X-DISCONTINUITY-SEQUENCE:1'
+    assert expected in result
+
 def test_should_correctly_update_base_path_if_its_blank():
     segment = Segment('entire.ts', 'http://1.2/')
     assert not segment.base_path
     segment.base_path = "base_path"
     assert "http://1.2/base_path/entire.ts" == segment.absolute_uri
 
+def test_base_path_should_just_return_uri_if_absolute():
+    segment = Segment('http://1.2/entire.ts', '')
+    assert 'http://1.2/entire.ts' == segment.absolute_uri
 
 def test_m3u8_should_propagate_base_uri_to_segments():
     with open(playlists.RELATIVE_PLAYLIST_FILENAME) as f:
@@ -775,6 +971,15 @@ def test_m3u8_should_propagate_base_uri_to_key():
     assert '../key.bin' == obj.keys[0].uri
     assert '/any/key.bin' == obj.keys[0].absolute_uri
 
+def test_m3u8_should_propagate_base_uri_to_session_key():
+    with open(playlists.RELATIVE_PLAYLIST_FILENAME) as f:
+        content = f.read()
+    obj = m3u8.M3U8(content, base_uri='/any/path')
+    assert '../key.bin' == obj.session_keys[0].uri
+    assert '/any/key.bin' == obj.session_keys[0].absolute_uri
+    obj.base_uri = '/any/where/'
+    assert '../key.bin' == obj.session_keys[0].uri
+    assert '/any/key.bin' == obj.session_keys[0].absolute_uri
 
 def test_base_path_with_optional_uri_should_do_nothing():
     media = Media(type='AUDIO', group_id='audio-group', name='English')
@@ -784,6 +989,16 @@ def test_base_path_with_optional_uri_should_do_nothing():
     assert media.absolute_uri is None
     assert media.base_path is None
 
+def test_medialist_uri_method():
+    langs = ['English', 'French', 'German']
+    ml = MediaList()
+    for lang in langs:
+        ml.append(Media(type='AUDIO', group_id='audio-group', name=lang, uri=('/%s.m3u8' % lang)))
+
+    assert len(ml.uri) == len(langs)
+    assert ml.uri[0] == '/%s.m3u8' % langs[0]
+    assert ml.uri[1] == '/%s.m3u8' % langs[1]
+    assert ml.uri[2] == '/%s.m3u8' % langs[2]
 
 def test_segment_map_uri_attribute():
     obj = m3u8.M3U8(playlists.MAP_URI_PLAYLIST)
@@ -824,7 +1039,154 @@ def test_should_dump_frame_rate():
 
     assert expected == obj.dumps().strip()
 
+@pytest.mark.skipif(sys.version_info >= (3,), reason="unicode not available in v3")
+def test_m3u8_unicode_method():
+    obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
 
+    result = unicode(obj).strip()
+    expected = playlists.SIMPLE_PLAYLIST.strip()
+    assert result == expected
+
+def test_add_segment_to_playlist():
+    obj = m3u8.M3U8()
+
+    obj.add_segment(
+        Segment(
+            'entire.ts',
+            'http://1.2/',
+            duration=1
+        )
+    )
+
+def test_segment_str_method():
+    segment = Segment('entire.ts', 'http://1.2/', duration=1)
+
+    expected = '#EXTINF:1,\nentire.ts'
+    result = str(segment).strip()
+
+    assert result == expected
+
+def test_attribute_denormaliser():
+    result = denormalize_attribute('test_test')
+    expected = 'TEST-TEST'
+
+    assert result == expected
+
+def test_find_key_throws_when_no_match():
+    threw = False
+    try:
+        find_key({
+            'method':   'AES-128',
+            'iv':       0x12345678,
+            'uri':      'http://1.2/'
+        }, [
+            # deliberately empty
+        ])
+    except KeyError as e:
+        threw = True
+    finally:
+        assert threw
+
+def test_ll_playlist():
+    obj = m3u8.M3U8(playlists.LOW_LATENCY_DELTA_UPDATE_PLAYLIST)
+    assert len(obj.rendition_reports) == 2
+    assert len(obj.segments[2].parts) == 12
+    print(obj.dumps())
+    assert (ext_x_part + ':DURATION=0.33334,URI="filePart271.0.ts"') in obj.dumps()
+
+def test_add_rendition_report_to_playlist():
+    obj = m3u8.M3U8()
+
+    obj.add_rendition_report(
+        RenditionReport(
+            base_uri='',
+            uri='../1M/waitForMSN.php',
+            last_msn=273,
+            last_part=0
+        )
+    )
+
+    result = obj.dumps()
+    expected = '#EXT-X-RENDITION-REPORT:URI="../1M/waitForMSN.php",LAST-MSN=273,LAST-PART=0'
+
+    assert expected in result
+
+def test_add_part_to_segment():
+    obj = Segment(
+        uri='fileSequence271.ts',
+        duration=4.00008
+    )
+
+    obj.add_part(
+        PartialSegment(
+            '',
+            'filePart271.0.ts',
+            0.33334
+        )
+    )
+
+    result = obj.dumps(None)
+    expected = '#EXT-X-PART:DURATION=0.33334,URI="filePart271.0.ts"'
+
+    assert expected in result
+
+def test_partial_segment_gap_and_byterange():
+    obj = PartialSegment(
+        '',
+        'filePart271.0.ts',
+        0.33334,
+        byterange='9400@376',
+        gap='YES'
+    )
+
+    result = obj.dumps(None)
+    expected = '#EXT-X-PART:DURATION=0.33334,URI="filePart271.0.ts",BYTERANGE=9400@376,GAP=YES'
+
+    assert result == expected
+
+def test_session_data_with_value():
+    obj = SessionData(
+        'com.example.value',
+        'example',
+        language='en'
+    )
+
+    result = obj.dumps()
+    expected = '#EXT-X-SESSION-DATA:DATA-ID="com.example.value",VALUE="example",LANGUAGE="en"'
+
+    assert result == expected
+
+def test_session_data_with_uri():
+    obj = SessionData(
+        'com.example.value',
+        uri='example.json',
+        language='en'
+    )
+
+    result = obj.dumps()
+    expected = '#EXT-X-SESSION-DATA:DATA-ID="com.example.value",URI="example.json",LANGUAGE="en"'
+
+    assert result == expected
+
+def test_session_data_cannot_be_created_with_value_and_uri_at_the_same_time():
+    obj = SessionData(
+        'com.example.value',
+        value='example',
+        uri='example.json',
+        language='en'
+    )
+
+    result = obj.dumps()
+    expected = '#EXT-X-SESSION-DATA:DATA-ID="com.example.value",VALUE="example",LANGUAGE="en"'
+
+    assert result == expected
+
+def test_endswith_newline():
+    obj = m3u8.loads(playlists.SIMPLE_PLAYLIST)
+
+    manifest = obj.dumps()
+
+    assert manifest.endswith('#EXT-X-ENDLIST\n')
 
 # custom asserts
 
