@@ -39,8 +39,8 @@ def test_should_parse_sliding_window_playlist_from_string():
             'https://priv.example.com/fileSequence2682.ts'] == [c['uri'] for c in data['segments']]
     assert [8, 8, 8] == [c['duration'] for c in data['segments']]
 
-def test_should_parse_playlist_with_encripted_segments_from_string():
-    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS)
+def test_should_parse_playlist_with_encrypted_segments_from_string():
+    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS)
     assert 7794 == data['media_sequence']
     assert 15 == data['targetduration']
     assert 'AES-128' == data['keys'][0]['method']
@@ -51,13 +51,13 @@ def test_should_parse_playlist_with_encripted_segments_from_string():
     assert [15, 15, 15] == [c['duration'] for c in data['segments']]
 
 def test_should_load_playlist_with_iv_from_string():
-    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
+    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV)
     assert "/hls-key/key.bin" == data['keys'][0]['uri']
     assert "AES-128" == data['keys'][0]['method']
     assert "0X10ef8f758ca555115584bb5b3c687f52" == data['keys'][0]['iv']
 
 def test_should_add_key_attribute_to_segment_from_playlist():
-    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS)
+    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS)
     first_segment_key = data['segments'][0]['key']
     assert "/hls-key/key.bin" == first_segment_key['uri']
     assert "AES-128" == first_segment_key['method']
@@ -91,11 +91,35 @@ def test_should_handle_key_method_none_and_no_uri_attr():
     assert "0X10ef8f758ca555115584bb5b3c687f52" == third_segment_key['iv']
     assert "NONE" == data['segments'][6]['key']['method']
 
-def test_should_parse_title_from_playlist():
-    data = m3u8.parse(playlists.SIMPLE_PLAYLIST_WITH_TITLE)
+def test_should_parse_playlist_with_session_encrypted_segments_from_string():
+    data = m3u8.parse(playlists.PLAYLIST_WITH_SESSION_ENCRYPTED_SEGMENTS)
+    assert 7794 == data['media_sequence']
+    assert 15 == data['targetduration']
+    assert 'AES-128' == data['session_keys'][0]['method']
+    assert 'https://priv.example.com/key.php?r=52' == data['session_keys'][0]['uri']
+    assert ['http://media.example.com/fileSequence52-1.ts',
+            'http://media.example.com/fileSequence52-2.ts',
+            'http://media.example.com/fileSequence52-3.ts'] == [c['uri'] for c in data['segments']]
+    assert [15, 15, 15] == [c['duration'] for c in data['segments']]
+
+def test_should_load_playlist_with_session_iv_from_string():
+    data = m3u8.parse(playlists.PLAYLIST_WITH_SESSION_ENCRYPTED_SEGMENTS_AND_IV)
+    assert "/hls-key/key.bin" == data['session_keys'][0]['uri']
+    assert "AES-128" == data['session_keys'][0]['method']
+    assert "0X10ef8f758ca555115584bb5b3c687f52" == data['session_keys'][0]['iv']
+
+def test_should_parse_quoted_title_from_playlist():
+    data = m3u8.parse(playlists.SIMPLE_PLAYLIST_WITH_QUOTED_TITLE)
     assert 1 == len(data['segments'])
     assert 5220 == data['segments'][0]['duration']
-    assert "A sample title" == data['segments'][0]['title']
+    assert '"A sample title"' == data['segments'][0]['title']
+    assert "http://media.example.com/entire.ts" == data['segments'][0]['uri']
+
+def test_should_parse_unquoted_title_from_playlist():
+    data = m3u8.parse(playlists.SIMPLE_PLAYLIST_WITH_UNQUOTED_TITLE)
+    assert 1 == len(data['segments'])
+    assert 5220 == data['segments'][0]['duration']
+    assert "A sample unquoted title" == data['segments'][0]['title']
     assert "http://media.example.com/entire.ts" == data['segments'][0]['uri']
 
 def test_should_parse_variant_playlist():
@@ -225,12 +249,12 @@ def test_should_parse_endlist_playlist():
     assert False == data['is_endlist']
 
 def test_should_parse_ALLOW_CACHE():
-    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
+    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV)
     assert 'no' == data['allow_cache']
 
 def test_should_parse_VERSION():
-    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
-    assert '2' == data['version']
+    data = m3u8.parse(playlists.PLAYLIST_WITH_ENCRYPTED_SEGMENTS_AND_IV)
+    assert 2 == data['version']
 
 def test_should_parse_program_date_time_from_playlist():
     data = m3u8.parse(playlists.SIMPLE_PLAYLIST_WITH_PROGRAM_DATE_TIME)
@@ -254,6 +278,11 @@ def test_should_parse_envivio_cue_playlist():
     assert '/DAlAAAENOOQAP/wFAUBAABrf+//N25XDf4B9p/gAAEBAQAAxKni9A==' == data['segments'][4]['scte35']
     assert '/DAlAAAENOOQAP/wFAUBAABrf+//N25XDf4B9p/gAAEBAQAAxKni9A==' == data['segments'][5]['scte35']
 
+def test_should_parse_no_duration_cue_playlist():
+    data = m3u8.parse(playlists.CUE_OUT_NO_DURATION_PLAYLIST)
+    assert data['segments'][0]['cue_out_start']
+    assert data['segments'][2]['cue_in']
+
 def test_parse_simple_playlist_messy():
     data = m3u8.parse(playlists.SIMPLE_PLAYLIST_MESSY)
     assert 5220 == data['targetduration']
@@ -289,6 +318,14 @@ def test_should_parse_segment_map_uri_with_byterange():
     assert data['segment_map']['uri'] == "main.mp4"
 
 
+def test_should_parse_multiple_map_attributes():
+    data = m3u8.parse(playlists.MULTIPLE_MAP_URI_PLAYLIST)
+
+    assert data['segments'][0]['init_section']['uri'] == 'init1.mp4'
+    assert data['segments'][1]['init_section']['uri'] == 'init1.mp4'
+    assert data['segments'][2]['init_section']['uri'] == 'init3.mp4'
+
+
 def test_should_parse_empty_uri_with_base_path():
     data = m3u8.M3U8(
             playlists.MEDIA_WITHOUT_URI_PLAYLIST,
@@ -310,6 +347,34 @@ def test_should_parse_start_with_precise():
     assert data['start']['time_offset'] == 10.5
     assert data['start']['precise'] == 'YES'
 
+
+def test_should_parse_session_data():
+    data = m3u8.parse(playlists.SESSION_DATA_PLAYLIST)
+    assert data['session_data'][0]['data_id'] == 'com.example.value'
+    assert data['session_data'][0]['value'] == 'example'
+    assert data['session_data'][0]['language'] == 'en'
+
+
+def test_should_parse_multiple_session_data():
+    data = m3u8.parse(playlists.MULTIPLE_SESSION_DATA_PLAYLIST)
+
+    assert len(data['session_data']) == 4
+
+    assert data['session_data'][0]['data_id'] == 'com.example.value'
+    assert data['session_data'][0]['value'] == 'example'
+    assert data['session_data'][0]['language'] == 'en'
+
+    assert data['session_data'][1]['data_id'] == 'com.example.value'
+    assert data['session_data'][1]['value'] == 'example'
+    assert data['session_data'][1]['language'] == 'ru'
+
+    assert data['session_data'][2]['data_id'] == 'com.example.value'
+    assert data['session_data'][2]['value'] == 'example'
+    assert data['session_data'][2]['language'] == 'de'
+
+    assert data['session_data'][3]['data_id'] == 'com.example.title'
+    assert data['session_data'][3]['uri'] == 'title.json'
+
 def test_simple_playlist_with_discontinuity_sequence():
     data = m3u8.parse(playlists.SIMPLE_PLAYLIST_WITH_DISCONTINUITY_SEQUENCE)
     assert data['discontinuity_sequence'] == 123
@@ -334,3 +399,23 @@ def test_master_playlist_with_frame_rate():
     assert 50 == playlists_list[1]['stream_info']['frame_rate']
     assert 60 == playlists_list[2]['stream_info']['frame_rate']
     assert 12.5 == playlists_list[3]['stream_info']['frame_rate']
+
+def test_low_latency_playlist():
+    data = m3u8.parse(playlists.LOW_LATENCY_DELTA_UPDATE_PLAYLIST)
+    assert data['server_control']['can_block_reload'] == 'YES'
+    assert data['server_control']['can_skip_until'] == 12.0
+    assert data['server_control']['part_hold_back'] == 1.0
+    assert data['part_inf']['part_target'] == 0.33334
+    assert data['skip']['skipped_segments'] == 3
+    assert len(data['segments'][2]['parts']) == 12
+    assert data['segments'][2]['parts'][0]['duration'] == 0.33334
+    assert data['segments'][2]['parts'][0]['uri'] == "filePart271.0.ts"
+    assert len(data['rendition_reports']) == 2
+    assert data['rendition_reports'][0]['uri'] == "../1M/waitForMSN.php"
+    assert data['rendition_reports'][0]['last_msn'] == 273
+    assert data['rendition_reports'][0]['last_part'] == 3
+
+
+def test_negative_media_sequence():
+    data = m3u8.parse(playlists.PLAYLIST_WITH_NEGATIVE_MEDIA_SEQUENCE)
+    assert data['media_sequence'] == -2680
