@@ -12,7 +12,7 @@ def test_create_a_variant_m3u8_with_two_playlists():
                            'English', 'YES', 'YES', 'NO', None)
     variant_m3u8.add_media(subtitles)
 
-    low_playlist = m3u8.Playlist('http://example.com/low.m3u8', stream_info={'bandwidth': 1280000, 'program_id': 1, 'subtitles': 'subs'}, media=[subtitles], base_uri=None)
+    low_playlist = m3u8.Playlist('http://example.com/low.m3u8', stream_info={'bandwidth': 1280000, 'program_id': 1, 'closed_captions': 'NONE', 'subtitles': 'subs'}, media=[subtitles], base_uri=None)
     high_playlist = m3u8.Playlist('http://example.com/high.m3u8', stream_info={'bandwidth': 3000000, 'program_id': 1, 'subtitles': 'subs'}, media=[subtitles], base_uri=None)
 
     variant_m3u8.add_playlist(low_playlist)
@@ -21,7 +21,7 @@ def test_create_a_variant_m3u8_with_two_playlists():
     expected_content = """\
 #EXTM3U
 #EXT-X-MEDIA:URI="english_sub.m3u8",TYPE=SUBTITLES,GROUP-ID="subs",LANGUAGE="en",NAME="English",DEFAULT=YES,AUTOSELECT=YES,FORCED=NO
-#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=1280000,SUBTITLES="subs"
+#EXT-X-STREAM-INF:PROGRAM-ID=1,CLOSED-CAPTIONS=NONE,BANDWIDTH=1280000,SUBTITLES="subs"
 http://example.com/low.m3u8
 #EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=3000000,SUBTITLES="subs"
 http://example.com/high.m3u8
@@ -58,6 +58,7 @@ def test_create_a_variant_m3u8_with_two_playlists_and_two_iframe_playlists():
         uri='video-800k-iframes.m3u8',
         iframe_stream_info={'bandwidth': 151288,
                             'program_id': 1,
+                            'closed_captions': None,
                             'resolution': '624x352',
                             'codecs': 'avc1.4d001f'},
         base_uri='http://example.com/'
